module KeyShot5

  @PLATFORM_IS_OSX = (Object::RUBY_PLATFORM =~ /darwin/i) ? true : false

  def self.PLATFORM_IS_OSX
    @PLATFORM_IS_OSX
  end

  @@KEYSHOT5_FOUND = false
  @@KEYSHOT51_FOUND = false
  @@TEXTURE_PATH = ""
  @@VERSION = ""

  def self.TEXTURE_PATH
    @@TEXTURE_PATH
  end

  if @PLATFORM_IS_OSX
    require 'luxion_keyshot5/mac/KeyShot5Export'
    require 'rexml/document'
  else
    if Sketchup.version_number > 14000000
      if(Sketchup.version.to_f >= 15 && Sketchup.is_64bit?)
        require 'luxion_keyshot5/win/64bit/KeyShot5Export'
      else
        require 'luxion_keyshot5/win/KeyShot5Export'
      end
    else
      require 'luxion_keyshot5/win/KeyShot5Export.dll'
    end
  end

  # Shim for the Set class which was moved in SketchUp 2014
  if defined?(Sketchup::Set)
    Set = Sketchup::Set
  end

  class KeyShot5Exporter
    include KeyShot5
    attr_accessor :ext_exporter

    def initialize
	    @texture_set = Set.new
    end

    # Visit an entity passing face information to the dll and recursing on groups and components
    def visit(entity, acc_trans, parent_mat, layer_name)
      useParentMat = false
      if not entity.hidden?
        if entity.is_a?(Sketchup::Face)
          if layer_name != ""
            if entity.layer.name != layer_name
              return
            end
          end

          material = nil
          frontmat = true
          if entity.material
            material = entity.material
          else
            if entity.back_material
              material = entity.back_material
              frontmat = false
            else
              if parent_mat
                material = parent_mat
			          useParentMat = true
              end
            end
          end

          materialname = ""
          texturename = ""
		      width = 1
		      height = 1
          if material
            materialname = material.display_name
            alpha = material.alpha
            color = material.color.to_a if material.color
            if material.texture
		          if useParentMat
			          width = 1.0 / material.texture.width
			          height = 1.0 / material.texture.height
			        end

              if File.exists?(material.texture.filename)
                texturename = material.texture.filename
              else
                texturename = material.texture.filename
                if texturename["/"]
                  texturename = texturename[texturename.rindex("/")+1..-1]
			          elsif texturename["\\"]
			            texturename = texturename[texturename.rindex("\\")+1..-1]
                end
			          # Texture writer converts .tga to .png - a pretty poorly documented feature
                texturename[texturename.size-3..-1] = "png" if texturename[texturename.size-3..-1] == "tga"
			          modelPath = Sketchup.active_model.path
			          filename = 'untitled_scene'
			          if File.exists?(modelPath)
				          filename = File.basename(modelPath, '.*')
			          end
                if KeyShot5.PLATFORM_IS_OSX
                  if TEXTURE_PATH != ""
                    texturename = TEXTURE_PATH + "/" + filename + "/" + texturename
                    dirname = TEXTURE_PATH + "/" + filename + "/"
                    unless File.exists?(dirname)
                      Dir.mkdir(dirname)
                    end
                  else
                    texturename = material.texture.filename
                  end
                else
                  if(ENV["KEYSHOT5"])
                    texturename = ENV["KEYSHOT5"] + "\\Textures\\" + filename + "\\" + texturename
                    dirname = ENV["KEYSHOT5"] + "\\Textures\\" + filename + "\\"
                    unless File.exists?(dirname)
                      Dir.mkdir(dirname)
                    end
                  else
                    texturename = material.texture.filename
                  end
                end

                unless File.exists?(texturename) and @texture_set.contains? texturename
                  @texture_set.insert texturename
                  puts texturename
                  # Add temporary face to export texture without transformations
                  model = Sketchup.active_model
                  entities = model.active_entities
                  pts = []
                  pts[0] = [0, 0, 0]
                  pts[1] = [1, 0, 0]
                  pts[2] = [1, 1, 0]
                  pts[3] = [0, 1, 0]
                  # Add the face to the entities in the model
                  tmp = entities.add_face pts
                  tmp.material = material
                  twtmp = Sketchup.create_texture_writer
                  twtmp.load(tmp, true)
                  twtmp.write(tmp, true, texturename)
                  entities.erase_entities([tmp])

                end
              end
            end
          else
            materialname = ""
            alpha = 1.0
            color = [255, 255, 255, 255]
            texturename = ""
          end

          # tell sketchup to make a triangle mesh from the current face
          # 5 as argument makes the mesh include all uvs and normals
          if frontmat
            mesh = entity.mesh 5
          else
            mesh = entity.mesh 6
          end

          # array of floats representing normal vectors
          narr = []
          i = 0
          (1..mesh.count_points).each { |i| narr = narr + mesh.normal_at(i).to_a}

        # call the export method in the dll
        color_a = color.to_a
        r = color_a[0]
        g = color_a[1]
        b = color_a[2]

        SUKeyShotExport.add_faces(mesh.points.length, mesh.polygons.length, mesh.points, mesh.polygons, narr, mesh.uvs(frontmat), materialname, texturename, r, g, b, alpha, width, height)

        end
        if entity.is_a?(Sketchup::ComponentInstance)
          if layer_name != ""
            if entity.layer.name != layer_name
              return
            end
          end
          transformation = entity.transformation
          name = ""
          if entity.name != ""
            name = entity.name
          else
            if entity.definition.name != ""
              name = entity.definition.name
            else
              if entity.material
                name = entity.material.display_name
              end
            end
          end
          SUKeyShotExport.begin_group(name);
          SUKeyShotExport.transform(transformation.to_a)
          entity.definition.entities.each do |e|
            visit(e, transformation, entity.material, "")
          end
          SUKeyShotExport.end_group
        end
        if entity.is_a?(Sketchup::Group)
          if layer_name != ""
            if entity.layer.name != layer_name
              return
            end
          end
          transformation = entity.transformation
          name = ""
          if entity.name != ""
            name = entity.name
          else
            if entity.material
              name = entity.material.display_name
            end
          end

          SUKeyShotExport.begin_group(name);
          SUKeyShotExport.transform(transformation.to_a)
          entity.entities.each do |e|
            visit(e, transformation, entity.material, "")
          end
          SUKeyShotExport.end_group

        end
      end
    end

    def ks5export(location, update)

      @texture_set.clear

      Sketchup.active_model.start_operation("task",true)
      model = Sketchup.active_model

      # File management
      filename = ""
      unless model.title == ''
        filename += model.title + '.bip'
      else
        filename += 'untitled.bip'
      end

      if(location != "")
        save = true
      else
        save = false
      end

      camera = model.active_view.camera

      position = camera.eye.to_a
      direction = camera.eye.vector_to(camera.target)
      up = camera.up
      fov = camera.fov
      cam_aspect = camera.aspect_ratio
      width = model.active_view.vpwidth
      height = model.active_view.vpheight

      if cam_aspect == 0.0
        cam_aspect = width/height.to_f.round(3)
      end

      if(update)
#        SUKeyShotExport.connect(filename, position, direction.to_a, camera.up.to_a, camera.fov, camera.aspect_ratio, width, height)
        SUKeyShotExport.connect(filename, position, direction.to_a, camera.up.to_a, fov, cam_aspect, width, height)
      else
#        SUKeyShotExport.open_file(filename, location, position, direction.to_a, camera.up.to_a, camera.fov, camera.aspect_ratio, save, width, height)
        SUKeyShotExport.open_file(filename, location, position, direction.to_a, camera.up.to_a, fov, cam_aspect, save, width, height)
      end

      idtrans = Geom::Transformation.new

      model.layers.each do |layer|
        if(layer.visible?)
          SUKeyShotExport.begin_group(layer.name)
          model.entities.each do |entity|
            visit(entity, idtrans, nil, layer.name)
          end

          SUKeyShotExport.end_group

        end
      end

      if(save)
        SUKeyShotExport.close_file
        result = UI.messagebox "Successfully exported to " + location, MB_OK
      else
        SUKeyShotExport.disconnect
      end
      model.commit_operation
    end # ksexport

  end #KeyShotExporter class


  #
  #   Set up UI hooks
  #

  if not file_loaded?(__FILE__)
    # Create the render and export menu items under Plugins->KeyShot 5
    unless @PLATFORM_IS_OSX and Sketchup.version.to_f < 14
      keyshot5_menu = UI.menu("Plugins").add_submenu("KeyShot 5")

      ks5_instance = KeyShot5Exporter.new

      if @PLATFORM_IS_OSX
        if File.exists?('/Library/Application Support/KeyShot5/keyshot5.xml')
          file = File.open('/Library/Application Support/KeyShot5/keyshot5.xml')
          xml = file.read
          doc = REXML::Document.new(xml)
          s = ""
          doc.elements.each("keyshot_configuration/folders/texture") { |element| s = element.text }
          TEXTURE_PATH = s

          doc.elements.each("keyshot_configuration") { |element| s = element.attributes["version"] }
          VERSION = s.split('.')[1]
          if(Sketchup.version.to_f > 14)
            @@KEYSHOT5_FOUND = true
            @@KEYSHOT51_FOUND = true
          else
            @@KEYSHOT5_FOUND = true
            if(@@VERSION >= 1)
              @@KEYSHOT51_FOUND = true
            end
          end
        end
      else
        if(ENV["KEYSHOT5"])
          @@KEYSHOT5_FOUND = true
          @@KEYSHOT51_FOUND = true
        end
      end

      if(@@KEYSHOT51_FOUND)
        keyshot5_menu.add_item("Render") do
          ks5_instance.ks5export("", false)
        end
      end

      keyshot5_menu.add_item("Export") do
        filename = ""
        unless Sketchup.active_model.title == ''
          filename += Sketchup.active_model.title + '.bip'
        else
          filename += 'untitled.bip'
        end
        if(ENV["KEYSHOT5"])
          keyshot5path = ENV["KEYSHOT5"] + "\\Scenes\\"
        else
          keyshot5path = ENV["HOME"]
        end
        location = UI.savepanel "Save KeyShot file", keyshot5path, filename

        if(location)
          ks5_instance.ks5export(location,false)
        end
      end

    # create the toolbar button

    if(@@KEYSHOT51_FOUND)
      renderCmd = UI::Command.new("Render") {
        ks5_instance.ks5export("", false)
      }
      renderCmd.small_icon = "keyshot_icon.png"
      renderCmd.large_icon = "keyshot_icon_large.png"
      renderCmd.tooltip = "Render in KeyShot 5"
      renderCmd.status_bar_text = "Render in KeyShot 5"
      renderCmd.menu_text = "Render in KeyShot 5"

      toolbar5 = UI::Toolbar.new "KeyShot 5"
      toolbar5 = toolbar5.add_item renderCmd
      toolbar5.show
    end
  end
  file_loaded(__FILE__)
end

end #KeyShot5 module
